! ESA (C) 2000-2018 
!
! This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
!
!    SAS is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    SAS is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
program checkoutput
!use realsame_mod
use dal
use errorhandling

implicit none

type(DataSetT)  :: inevents
type(TableT)    :: evtable
type(ColumnT)   :: xCol, yCol 
integer(int32), dimension(:), pointer :: p_x, p_y
integer nrows, off
logical ok

ok = .true. 

off = 2 		!offset for co-ords

!-------------------------------------------------------------------------
!	PN
write (*,*) 'testing PN'
inevents = dataSet("pn.in", read)
evtable = table(inevents,"EVENTS")
if (.not.hasColumn(evtable,"X") .or. &
    .not.hasColumn(evtable,"Y")) ok = .false.
if (.not.ok) call error("noColumn","PN X and/or Y Column absent or incorrectly named")
ok = .true.
xCol = column(evtable,"X",read)
yCol = column(evtable,"Y",read)
p_x => int32Data(xCol)
p_y => int32Data(yCol)
if (p_x(0) < p_x(2)) ok = .false.
if (.not.ok) call error("wrongCoords","PN X and/or Y coords. incorrect")
ok = .true.

call release(evtable)
call release(inevents)
!-------------------------------------------------------------------------
!	MOS1
write (*,*) 'testing MOS1'
inevents = dataSet("mos1.in", read)
evtable = table(inevents,"EVENTS")
if (.not.hasColumn(evtable,"X") .or. &
    .not.hasColumn(evtable,"Y")) ok = .false.
if (.not.ok) call error("noColumn","MOS1 X and/or Y Column absent or incorrectly named")
ok = .true.
xCol = column(evtable,"X",read)
yCol = column(evtable,"Y",read)
p_x => int32Data(xCol)
p_y => int32Data(yCol)
if (p_y(0) < p_y(3)) ok = .false.
if (.not.ok) call error("wrongCoords","MOS1 X and/or Y coords. incorrect")
ok = .true.

call release(evtable)
call release(inevents)
!-------------------------------------------------------------------------
!	MOS2
write (*,*) 'testing MOS2'
inevents = dataSet("mos2.in", read)
evtable = table(inevents,"EVENTS")
if (.not.hasColumn(evtable,"X") .or. &
    .not.hasColumn(evtable,"Y")) ok = .false.
if (.not.ok) call error("noColumn","MOS2 X and/or Y Column absent or incorrectly named")
ok = .true.
xCol = column(evtable,"X",read)
yCol = column(evtable,"Y",read)
p_x => int32Data(xCol)
p_y => int32Data(yCol)
if (numberofRows(xCol) /= 319) ok = .false.
if (.not.ok) call error("wrongCoords","MOS2 X and/or Y coords. incorrect")
ok = .true.

call release(evtable)
call release(inevents)
!--------------------------------------------------------------------------
!	PN L
write (*,*) 'testing PN B/sight and orientaion'
inevents = dataSet("pn.in.L", read)
evtable = table(inevents,"EVENTS")
xCol = column(evtable,"X",read)
yCol = column(evtable,"Y",read)
p_x => int32Data(xCol)
p_y => int32Data(yCol)
nrows = numberofRows(evtable)

if (p_y(0) > p_y(1)) ok = .false.
if (p_x(2) < p_x(1)) ok = .false.
if (.not.ok) call error("wrongOrientation","PN Orientation incorrect")
ok = .true.

call release(evtable)
call release(inevents)
!-------------------------------------------------------------------------
!--------------------------------------------------------------------------
!	MOS1 L
write (*,*) 'testing MOS1 B/sight and orientaion'
inevents = dataSet("mos1.in.L", read)
evtable = table(inevents,"EVENTS")
xCol = column(evtable,"X",read)
yCol = column(evtable,"Y",read)
p_x => int32Data(xCol)
p_y => int32Data(yCol)
nrows = numberofRows(evtable)

if (p_x(0) < p_x(1)) ok = .false.
if (p_y(2) < p_y(1)) ok = .false.
if (.not.ok) call error("wrongOrientation","MOS1 Orientation incorrect")
ok = .true.

call release(evtable)
call release(inevents)
!-------------------------------------------------------------------------
!--------------------------------------------------------------------------
!	MOS2 L
write (*,*) 'testing MOS2 B/sight and orientaion'
inevents = dataSet("mos2.in.L", read)
evtable = table(inevents,"EVENTS")
xCol = column(evtable,"X",read)
yCol = column(evtable,"Y",read)
p_x => int32Data(xCol)
p_y => int32Data(yCol)
nrows = numberofRows(evtable)

if (p_y(0) < p_y(1)) ok = .false.
if (p_x(2) > p_x(1)) ok = .false.
if (.not.ok) call error("wrongOrientation","MOS2 Orientation incorrect")
ok = .true.

call release(evtable)
call release(inevents)
!-------------------------------------------------------------------------
!-------------------------------------------------------------------------
!	PN (AHF/PNT)
write (*,*) 'testing moving PN with AHF and PNT'
inevents = dataSet("pn.in_mv", read)
evtable = table(inevents,"EVENTS")
if (.not.hasColumn(evtable,"X") .or. &
    .not.hasColumn(evtable,"Y")) ok = .false.
if (.not.ok) call error("noColumn","PN(mv) X and/or Y Column absent or incorrectly named")
ok = .true.
xCol = column(evtable,"X",read)
yCol = column(evtable,"Y",read)
p_x => int32Data(xCol)
p_y => int32Data(yCol)
if (p_x(0) < p_y(0)) ok = .false.
if (p_x(3) < p_y(3)) ok = .false.
if (.not.ok) call error("wrongCoords","PN(mv) X and/or Y coords. incorrect")
ok = .true.

call release(evtable)
call release(inevents)
!-------------------------------------------------------------------------
!	MOS1 (AHF/PNT)
write (*,*) 'testing moving MOS1 with AHF and PNT'
inevents = dataSet("mos1.in_mv", read)
evtable = table(inevents,"EVENTS")
if (.not.hasColumn(evtable,"X") .or. &
    .not.hasColumn(evtable,"Y")) ok = .false.
if (.not.ok) call error("noColumn","MOS1(mv) X and/or Y Column absent or incorrectly named")
ok = .true.
xCol = column(evtable,"X",read)
yCol = column(evtable,"Y",read)
p_x => int32Data(xCol)
p_y => int32Data(yCol)
if (p_y(0) < p_y(3)) ok = .false.
if (.not.ok) call error("wrongCoords","MOS1(mv) X and/or Y coords. incorrect")
ok = .true.

call release(evtable)
call release(inevents)
!-------------------------------------------------------------------------
!-------------------------------------------------------------------------
!	PN (AHF/PNT) NULL testing
write (*,*) 'bad attitude/NULL PN with AHF and PNT'
inevents = dataSet("pn.in.odfbad", read)
evtable = table(inevents,"EVENTS")
if (.not.hasColumn(evtable,"X") .or. &
    .not.hasColumn(evtable,"Y")) ok = .false.
if (.not.ok) call error("noColumn","PN(mv) X and/or Y Column absent or incorrectly named")
ok = .true.
xCol = column(evtable,"X",read)
yCol = column(evtable,"Y",read)
p_x => int32Data(xCol)
p_y => int32Data(yCol)

if (.not.isNull(xCol,0)) ok = .false.
if (.not.ok) call error("notNullXcol","X values are not NULL")
ok = .true.
if (isNull(xCol,0)) write (*,*) "X NULL found - good"
if (.not.isNull(yCol,0)) ok = .false.
if (.not.ok) call error("notNullYcol","Y values are not NULL")
ok = .true.
if (isNull(yCol,0)) write (*,*) "Y NULL found - good"

call release(evtable)
call release(inevents)
!-------------------------------------------------------------------------

end

